@@ -7,7 +7,7 @@ mod tests;
 pub mod visit;
 
 pub use self::typed::TypedExpr;
-pub use self::untyped::{UntypedExpr, Use};
+pub use self::untyped::{FunctionLiteralKind, UntypedExpr, Use};
 
 pub use self::constant::{Constant, TypedConstant, UntypedConstant};
 
@@ -15,17 +15,18 @@ use crate::analyse::Inferred;
 use crate::build::{Located, Target};
 use crate::parse::SpannedString;
 use crate::type_::expression::Implementations;
+use crate::type_::printer::Names;
 use crate::type_::{
     self, Deprecation, ModuleValueConstructor, PatternConstructor, Type, ValueConstructor,
 };
 use std::sync::Arc;
 
 use ecow::EcoString;
+use num_bigint::BigInt;
 #[cfg(test)]
 use pretty_assertions::assert_eq;
 use vec1::Vec1;
 
-pub const TRY_VARIABLE: &str = "_try";
 pub const PIPE_VARIABLE: &str = "_pipe";
 pub const USE_ASSIGNMENT_VARIABLE: &str = "_use";
 pub const ASSERT_FAIL_VARIABLE: &str = "_assert_fail";
@@ -46,6 +47,7 @@ pub struct Module<Info, Statements> {
     pub documentation: Vec<EcoString>,
     pub type_info: Info,
     pub definitions: Vec<Statements>,
+    pub names: Names,
 }
 
 impl TypedModule {
@@ -168,6 +170,11 @@ impl<A> Arg<A> {
 impl TypedArg {
     pub fn find_node(&self, byte_index: u32) -> Option<Located<'_>> {
         if self.location.contains(byte_index) {
+            if let Some(annotation) = &self.annotation {
+                return annotation
+                    .find_node(byte_index, self.type_.clone())
+                    .or(Some(Located::Arg(self)));
+            }
             Some(Located::Arg(self))
         } else {
             None
@@ -692,6 +699,7 @@ impl<T> Import<T> {
 }
 
 pub type UntypedModuleConstant = ModuleConstant<(), ()>;
+pub type TypedModuleConstant = ModuleConstant<Arc<Type>, EcoString>;
 
 #[derive(Debug, Clone, PartialEq, Eq)]
 /// A certain fixed value that can be used in multiple places
@@ -826,15 +834,9 @@ impl TypedDefinition {
                 if let Some(found_arg) = function
                     .arguments
                     .iter()
-                    .find(|arg| arg.location.contains(byte_index))
+                    .find_map(|arg| arg.find_node(byte_index))
                 {
-                    // Check if location is within the arg annotation.
-                    if let Some(a) = &found_arg.annotation {
-                        return a
-                            .find_node(byte_index, found_arg.type_.clone())
-                            .or(Some(Located::Arg(found_arg)));
-                    }
-                    return Some(Located::Arg(found_arg));
+                    return Some(found_arg);
                 };
 
                 if let Some(found_statement) = function
@@ -1283,7 +1285,7 @@ impl<T> HasLocation for CallArg<T> {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub struct RecordUpdateSpread {
+pub struct RecordBeingUpdated {
     pub base: Box<UntypedExpr>,
     pub location: SrcSpan,
 }
@@ -1676,6 +1678,7 @@ pub enum Pattern<Type> {
     Int {
         location: SrcSpan,
         value: EcoString,
+        int_value: BigInt,
     },
 
     Float {
@@ -1951,6 +1954,8 @@ pub enum AssignmentKind {
     Let,
     // let assert x = ...
     Assert { location: SrcSpan },
+    // For assignments generated by the compiler
+    Generated,
 }
 
 impl AssignmentKind {
@@ -1961,7 +1966,17 @@ impl AssignmentKind {
     pub fn is_assert(&self) -> bool {
         match self {
             Self::Assert { .. } => true,
-            Self::Let => false,
+            Self::Let | Self::Generated => false,
+        }
+    }
+    /// Returns `true` if the assignment kind is [`Generated`].
+    ///
+    /// [`Generated`]: AssignmentKind::Generated
+    #[must_use]
+    pub fn is_generated(&self) -> bool {
+        match self {
+            Self::Generated => true,
+            Self::Let | Self::Assert { .. } => false,
         }
     }
 }
