@@ -9,6 +9,7 @@ pub enum TypedExpr {
         location: SrcSpan,
         type_: Arc<Type>,
         value: EcoString,
+        int_value: BigInt,
     },
 
     Float {
@@ -48,7 +49,7 @@ pub enum TypedExpr {
     Fn {
         location: SrcSpan,
         type_: Arc<Type>,
-        is_capture: bool,
+        kind: FunctionLiteralKind,
         args: Vec<TypedArg>,
         body: Vec1<TypedStatement>,
         return_annotation: Option<TypeAst>,
@@ -135,7 +136,7 @@ pub enum TypedExpr {
     RecordUpdate {
         location: SrcSpan,
         type_: Arc<Type>,
-        spread: Box<Self>,
+        record: Box<Self>,
         args: Vec<TypedRecordUpdateArg>,
     },
 
@@ -221,9 +222,23 @@ impl TypedExpr {
             // beyond the index under search.
             Self::Tuple {
                 elems: expressions, ..
+            } => {
+                for expression in expressions {
+                    if expression.location().start > byte_index {
+                        break;
+                    }
+
+                    if let Some(located) = expression.find_node(byte_index) {
+                        return Some(located);
+                    }
+                }
+
+                self.self_if_contains_location(byte_index)
             }
-            | Self::List {
+
+            Self::List {
                 elements: expressions,
+                tail,
                 ..
             } => {
                 for expression in expressions {
@@ -236,6 +251,11 @@ impl TypedExpr {
                     }
                 }
 
+                if let Some(tail) = tail {
+                    if let Some(node) = tail.find_node(byte_index) {
+                        return Some(node);
+                    }
+                }
                 self.self_if_contains_location(byte_index)
             }
 
@@ -281,10 +301,10 @@ impl TypedExpr {
                 .find_map(|arg| arg.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
 
-            Self::RecordUpdate { spread, args, .. } => args
+            Self::RecordUpdate { record, args, .. } => args
                 .iter()
                 .find_map(|arg| arg.find_node(byte_index))
-                .or_else(|| spread.find_node(byte_index))
+                .or_else(|| record.find_node(byte_index))
                 .or_else(|| self.self_if_contains_location(byte_index)),
         }
     }
@@ -524,16 +544,10 @@ impl TypedExpr {
                 value.is_pure_value_constructor()
             }
 
-            // A module select is a pure value constructor only if it is a
-            // record, in all other cases it could be a side-effecting function.
-            // For example `option.Some(1)` is pure but `io.println("a")` is
-            // not.
-            TypedExpr::ModuleSelect { constructor, .. } => match constructor {
-                ModuleValueConstructor::Record { .. } => true,
-                ModuleValueConstructor::Fn { .. } | ModuleValueConstructor::Constant { .. } => {
-                    false
-                }
-            },
+            // Just selecting a value from a module never has any effects. The
+            // selected thing might be a function but it has no side effects as
+            // long as it's not called!
+            TypedExpr::ModuleSelect { .. } => true,
 
             // A pipeline is a pure value constructor if its last step is a record builder.
             // For example `wibble() |> wobble() |> Ok`
