@@ -20,7 +20,7 @@ use crate::{
     },
     Result,
 };
-use ecow::EcoString;
+use ecow::{eco_format, EcoString};
 use heck::ToSnakeCase;
 use im::HashSet;
 use itertools::Itertools;
@@ -34,7 +34,7 @@ const INDENT: isize = 4;
 const MAX_COLUMNS: isize = 80;
 
 fn module_name_to_erlang(module: &str) -> Document<'_> {
-    Document::String(module.replace('/', "@"))
+    EcoString::from(module.replace('/', "@")).to_doc()
 }
 
 fn module_name_atom(module: &str) -> Document<'static> {
@@ -67,14 +67,14 @@ impl<'env> Env<'env> {
             None => {
                 let _ = self.current_scope_vars.insert(name.to_string(), 0);
                 let _ = self.erl_function_scope_vars.insert(name.to_string(), 0);
-                Document::String(variable_name(name))
+                variable_name(name).to_doc()
             }
-            Some(0) => Document::String(variable_name(name)),
+            Some(0) => variable_name(name).to_doc(),
             Some(n) => {
                 use std::fmt::Write;
                 let mut name = variable_name(name);
                 write!(name, "@{n}").expect("pushing number suffix to name");
-                Document::String(name)
+                name.to_doc()
             }
         }
     }
@@ -162,7 +162,7 @@ fn module_document<'a>(
 
     let header = "-module("
         .to_doc()
-        .append(Document::String(module.name.replace("/", "@").to_string()))
+        .append(module.name.replace("/", "@"))
         .append(").")
         .append(line());
 
@@ -298,7 +298,8 @@ fn register_imports(
             };
             // Type Exports
             type_exports.push(
-                Document::String(erl_safe_type_name(name.to_snake_case()))
+                erl_safe_type_name(name.to_snake_case())
+                    .to_doc()
                     .append("/")
                     .append(typed_parameters.len()),
             );
@@ -331,7 +332,7 @@ fn register_imports(
             );
             let doc = if *opaque { "-opaque " } else { "-type " }
                 .to_doc()
-                .append(Document::String(erl_safe_type_name(name.to_snake_case())))
+                .append(erl_safe_type_name(name.to_snake_case()))
                 .append("(")
                 .append(params)
                 .append(") :: ")
@@ -404,7 +405,11 @@ fn module_function<'a>(
     let return_spec = type_printer.print(&function.return_type);
 
     let spec = fun_spec(function_name, args_spec, return_spec);
-    let arguments = fun_args(&function.arguments, &mut env);
+    let arguments = if function.external_erlang.is_some() {
+        external_fun_args(&function.arguments, &mut env)
+    } else {
+        fun_args(&function.arguments, &mut env)
+    };
 
     let body = function
         .external_erlang
@@ -441,6 +446,22 @@ fn file_attribute<'a>(
     docvec!["-file(\"", path, "\", ", line, ")."]
 }
 
+fn external_fun_args<'a>(args: &'a [TypedArg], env: &mut Env<'a>) -> Document<'a> {
+    wrap_args(args.iter().map(|a| {
+        let name = match &a.names {
+            ArgNames::Discard { name, .. }
+            | ArgNames::LabelledDiscard { name, .. }
+            | ArgNames::Named { name, .. }
+            | ArgNames::NamedLabelled { name, .. } => name,
+        };
+        if name.chars().all(|c| c == '_') {
+            env.next_local_var_name("argument")
+        } else {
+            env.next_local_var_name(name)
+        }
+    }))
+}
+
 fn fun_args<'a>(args: &'a [TypedArg], env: &mut Env<'a>) -> Document<'a> {
     wrap_args(args.iter().map(|a| match &a.names {
         ArgNames::Discard { .. } | ArgNames::LabelledDiscard { .. } => "_".to_doc(),
@@ -479,7 +500,7 @@ fn fun_spec<'a>(
 }
 
 fn atom_string(value: String) -> Document<'static> {
-    Document::String(escape_atom_string(value))
+    escape_atom_string(value).to_doc()
 }
 
 fn atom_pattern() -> &'static Regex {
@@ -490,26 +511,26 @@ fn atom_pattern() -> &'static Regex {
 fn atom(value: &str) -> Document<'_> {
     if is_erlang_reserved_word(value) {
         // Escape because of keyword collision
-        Document::String(format!("'{value}'"))
+        eco_format!("'{value}'").to_doc()
     } else if atom_pattern().is_match(value) {
         // No need to escape
-        Document::Str(value)
+        EcoString::from(value).to_doc()
     } else {
         // Escape because of characters contained
-        Document::String(format!("'{value}'"))
+        eco_format!("'{value}'").to_doc()
     }
 }
 
-fn escape_atom_string(value: String) -> String {
+fn escape_atom_string(value: String) -> EcoString {
     if is_erlang_reserved_word(&value) {
         // Escape because of keyword collision
-        format!("'{value}'")
+        eco_format!("'{value}'")
     } else if atom_pattern().is_match(&value) {
         // No need to escape
-        value
+        EcoString::from(value)
     } else {
         // Escape because of characters contained
-        format!("'{value}'")
+        eco_format!("'{value}'")
     }
 }
 
@@ -533,9 +554,8 @@ fn string_inner(value: &str) -> Document<'_> {
             } else {
                 format!("{slashes}x")
             }
-        })
-        .to_string();
-    Document::String(content)
+        });
+    EcoString::from(content).to_doc()
 }
 
 fn string(value: &str) -> Document<'_> {
@@ -669,21 +689,21 @@ fn const_segment<'a>(
     options: &'a [TypedConstantBitArraySegmentOption],
     env: &mut Env<'a>,
 ) -> Document<'a> {
-    let mut value_is_a_string_literal = false;
-    let document = match value {
-        // Skip the normal <<value/utf8>> surrounds
-        Constant::String { value, .. } => {
-            value_is_a_string_literal = true;
-            value.to_doc().surround("\"", "\"")
-        }
+    let value_is_a_string_literal = matches!(value, Constant::String { .. });
 
-        // As normal
-        Constant::Int { .. } | Constant::Float { .. } | Constant::BitArray { .. } => {
-            const_inline(value, env)
-        }
+    let create_document = |env: &mut Env<'a>| {
+        match value {
+            // Skip the normal <<value/utf8>> surrounds
+            Constant::String { value, .. } => value.to_doc().surround("\"", "\""),
 
-        // Wrap anything else in parentheses
-        value => const_inline(value, env).surround("(", ")"),
+            // As normal
+            Constant::Int { .. } | Constant::Float { .. } | Constant::BitArray { .. } => {
+                const_inline(value, env)
+            }
+
+            // Wrap anything else in parentheses
+            value => const_inline(value, env).surround("(", ")"),
+        }
     };
 
     let size = |value: &'a TypedConstant, env: &mut Env<'a>| match value {
@@ -694,10 +714,10 @@ fn const_segment<'a>(
         ),
     };
 
-    let unit = |value: &'a u8| Some(Document::String(format!("unit:{value}")));
+    let unit = |value: &'a u8| Some(eco_format!("unit:{value}").to_doc());
 
     bit_array_segment(
-        document,
+        create_document,
         options,
         size,
         unit,
@@ -722,30 +742,29 @@ fn expr_segment<'a>(
     options: &'a [BitArrayOption<TypedExpr>],
     env: &mut Env<'a>,
 ) -> Document<'a> {
-    let mut value_is_a_string_literal = false;
+    let value_is_a_string_literal = matches!(value, TypedExpr::String { .. });
 
-    let document = match value {
-        // Skip the normal <<value/utf8>> surrounds and set the string literal flag
-        TypedExpr::String { value, .. } => {
-            value_is_a_string_literal = true;
-            string_inner(value).surround("\"", "\"")
+    let create_document = |env: &mut Env<'a>| {
+        match value {
+            // Skip the normal <<value/utf8>> surrounds and set the string literal flag
+            TypedExpr::String { value, .. } => string_inner(value).surround("\"", "\""),
+
+            // As normal
+            TypedExpr::Int { .. }
+            | TypedExpr::Float { .. }
+            | TypedExpr::Var { .. }
+            | TypedExpr::BitArray { .. } => expr(value, env),
+
+            // Wrap anything else in parentheses
+            value => expr(value, env).surround("(", ")"),
         }
-
-        // As normal
-        TypedExpr::Int { .. }
-        | TypedExpr::Float { .. }
-        | TypedExpr::Var { .. }
-        | TypedExpr::BitArray { .. } => expr(value, env),
-
-        // Wrap anything else in parentheses
-        value => expr(value, env).surround("(", ")"),
     };
 
     let size = |expression: &'a TypedExpr, env: &mut Env<'a>| match expression {
         TypedExpr::Int { value, .. } => {
             let v = value.replace("_", "");
             let v = u64::from_str(&v).unwrap_or(0);
-            Some(Document::String(format!(":{v}")))
+            Some(eco_format!(":{v}").to_doc())
         }
 
         _ => {
@@ -761,10 +780,10 @@ fn expr_segment<'a>(
         }
     };
 
-    let unit = |value: &'a u8| Some(Document::String(format!("unit:{value}")));
+    let unit = |value: &'a u8| Some(eco_format!("unit:{value}").to_doc());
 
     bit_array_segment(
-        document,
+        create_document,
         options,
         size,
         unit,
@@ -774,8 +793,8 @@ fn expr_segment<'a>(
     )
 }
 
-fn bit_array_segment<'a, Value: 'a, SizeToDoc, UnitToDoc>(
-    mut document: Document<'a>,
+fn bit_array_segment<'a, Value: 'a, CreateDoc, SizeToDoc, UnitToDoc>(
+    mut create_document: CreateDoc,
     options: &'a [BitArrayOption<Value>],
     mut size_to_doc: SizeToDoc,
     mut unit_to_doc: UnitToDoc,
@@ -784,6 +803,7 @@ fn bit_array_segment<'a, Value: 'a, SizeToDoc, UnitToDoc>(
     env: &mut Env<'a>,
 ) -> Document<'a>
 where
+    CreateDoc: FnMut(&mut Env<'a>) -> Document<'a>,
     SizeToDoc: FnMut(&'a Value, &mut Env<'a>) -> Option<Document<'a>>,
     UnitToDoc: FnMut(&'a u8) -> Option<Document<'a>>,
 {
@@ -826,6 +846,8 @@ where
         }
     }
 
+    let mut document = create_document(env);
+
     document = document.append(size);
     let others_is_empty = others.is_empty();
 
@@ -1042,9 +1064,9 @@ fn float<'a>(value: &str) -> Document<'a> {
     match value.split('.').collect_vec().as_slice() {
         ["0", "0"] => "+0.0".to_doc(),
         [before_dot, after_dot] if after_dot.starts_with('e') => {
-            Document::String(format!("{before_dot}.0{after_dot}"))
+            eco_format!("{before_dot}.0{after_dot}").to_doc()
         }
-        _ => Document::String(value),
+        _ => EcoString::from(value).to_doc(),
     }
 }
 
@@ -1081,11 +1103,11 @@ fn var<'a>(name: &'a str, constructor: &'a ValueConstructor, env: &mut Env<'a>)
                 let chars = incrementing_args_list(args.len());
                 "fun("
                     .to_doc()
-                    .append(Document::String(chars.clone()))
+                    .append(chars.clone())
                     .append(") -> {")
                     .append(atom_string(record_name.to_snake_case()))
                     .append(", ")
-                    .append(Document::String(chars))
+                    .append(chars)
                     .append("} end")
             }
             _ => atom_string(record_name.to_snake_case()),
@@ -1141,7 +1163,7 @@ fn int<'a>(value: &str) -> Document<'a> {
         value.replace_range(..2, "2#");
     }
 
-    Document::String(value)
+    EcoString::from(value).to_doc()
 }
 
 fn const_inline<'a>(literal: &'a TypedConstant, env: &mut Env<'a>) -> Document<'a> {
@@ -1200,11 +1222,11 @@ fn record_constructor_function(tag: &EcoString, arity: usize) -> Document<'_> {
     let chars = incrementing_args_list(arity);
     "fun("
         .to_doc()
-        .append(Document::String(chars.clone()))
+        .append(chars.clone())
         .append(") -> {")
         .append(atom_string(tag.to_snake_case()))
         .append(", ")
-        .append(Document::String(chars))
+        .append(chars)
         .append("} end")
 }
 
@@ -1401,7 +1423,7 @@ fn tuple_index_inline<'a>(
     index: u64,
     env: &mut Env<'a>,
 ) -> Document<'a> {
-    let index_doc = Document::String(format!("{}", (index + 1)));
+    let index_doc = eco_format!("{}", (index + 1)).to_doc();
     let tuple_doc = bare_clause_guard(tuple, env);
     "erlang:element"
         .to_doc()
@@ -1594,11 +1616,7 @@ fn docs_args_call<'a>(
                 .append(args)
         }
 
-        TypedExpr::Fn {
-            is_capture: true,
-            body,
-            ..
-        } => {
+        TypedExpr::Fn { kind, body, .. } if kind.is_capture() => {
             if let Statement::Expression(TypedExpr::Call {
                 fun,
                 args: inner_args,
@@ -1638,11 +1656,11 @@ fn docs_args_call<'a>(
 }
 
 fn record_update<'a>(
-    spread: &'a TypedExpr,
+    record: &'a TypedExpr,
     args: &'a [TypedRecordUpdateArg],
     env: &mut Env<'a>,
 ) -> Document<'a> {
-    let expr_doc = maybe_block_expr(spread, env);
+    let expr_doc = maybe_block_expr(record, env);
 
     args.iter().fold(expr_doc, |tuple_doc, arg| {
         // Increment the index by 2, because the first element
@@ -1826,7 +1844,7 @@ fn expr<'a>(expression: &'a TypedExpr, env: &mut Env<'a>) -> Document<'a> {
 
         TypedExpr::RecordAccess { record, index, .. } => tuple_index(record, index + 1, env),
 
-        TypedExpr::RecordUpdate { spread, args, .. } => record_update(spread, args, env),
+        TypedExpr::RecordUpdate { record, args, .. } => record_update(record, args, env),
 
         TypedExpr::Case {
             subjects, clauses, ..
@@ -1867,7 +1885,9 @@ fn pipeline<'a>(
 
 fn assignment<'a>(assignment: &'a TypedAssignment, env: &mut Env<'a>) -> Document<'a> {
     match assignment.kind {
-        AssignmentKind::Let => let_(&assignment.value, &assignment.pattern, env),
+        AssignmentKind::Let | AssignmentKind::Generated => {
+            let_(&assignment.value, &assignment.pattern, env)
+        }
         AssignmentKind::Assert { .. } => let_assert(&assignment.value, &assignment.pattern, env),
     }
 }
@@ -1877,7 +1897,7 @@ fn negate_with<'a>(op: &'static str, value: &'a TypedExpr, env: &mut Env<'a>) ->
 }
 
 fn tuple_index<'a>(tuple: &'a TypedExpr, index: u64, env: &mut Env<'a>) -> Document<'a> {
-    let index_doc = Document::String(format!("{}", (index + 1)));
+    let index_doc = eco_format!("{}", (index + 1)).to_doc();
     let tuple_doc = maybe_block_expr(tuple, env);
     "erlang:element"
         .to_doc()
@@ -1918,17 +1938,19 @@ fn fun<'a>(args: &'a [TypedArg], body: &'a [TypedStatement], env: &mut Env<'a>)
     doc
 }
 
-fn incrementing_args_list(arity: usize) -> String {
+fn incrementing_args_list(arity: usize) -> EcoString {
     let arguments = (0..arity).map(|c| format!("Field@{c}"));
-    Itertools::intersperse(arguments, ", ".into()).collect()
+    Itertools::intersperse(arguments, ", ".into())
+        .collect::<String>()
+        .into()
 }
 
-fn variable_name(name: &str) -> String {
+fn variable_name(name: &str) -> EcoString {
     let mut chars = name.chars();
     let first_char = chars.next();
     let first_uppercased = first_char.into_iter().flat_map(char::to_uppercase);
 
-    first_uppercased.chain(chars).collect()
+    first_uppercased.chain(chars).collect::<EcoString>()
 }
 
 /// When rendering a type variable to an erlang type spec we need all type variables with the
@@ -1936,9 +1958,9 @@ fn variable_name(name: &str) -> String {
 /// This function converts a usize into base 26 A-Z for this purpose.
 fn id_to_type_var(id: u64) -> Document<'static> {
     if id < 26 {
-        let mut name = "".to_string();
+        let mut name = EcoString::from("");
         name.push(char::from_u32((id % 26 + 65) as u32).expect("id_to_type_var 0"));
-        return Document::String(name);
+        return name.to_doc();
     }
     let mut name = vec![];
     let mut last_char = id;
@@ -2148,7 +2170,7 @@ fn type_var_ids(type_: &Type, ids: &mut HashMap<u64, u64>) {
     }
 }
 
-fn erl_safe_type_name(mut name: String) -> String {
+fn erl_safe_type_name(mut name: String) -> EcoString {
     if matches!(
         name.as_str(),
         "any"
@@ -2191,7 +2213,7 @@ fn erl_safe_type_name(mut name: String) -> String {
             | "tuple"
     ) {
         name.push('_');
-        name
+        EcoString::from(name)
     } else {
         escape_atom_string(name)
     }
@@ -2281,7 +2303,7 @@ impl<'a> TypePrinter<'a> {
 
     fn print_type_app(&self, module: &str, name: &str, args: &[Arc<Type>]) -> Document<'static> {
         let args = join(args.iter().map(|a| self.print(a)), ", ".to_doc());
-        let name = Document::String(erl_safe_type_name(name.to_snake_case()));
+        let name = erl_safe_type_name(name.to_snake_case()).to_doc();
         if self.current_module == module {
             docvec![name, "(", args, ")"]
         } else {
