@@ -52,7 +52,8 @@ fn module_types() {
                 package: "package".into(),
                 module: module_name,
                 name: "my_type".into(),
-                args: vec![]
+                args: vec![],
+                inferred_variant: None,
             })
         )]
     ));
