@@ -235,3 +235,35 @@ pub fn main() {
 "#
     )
 }
+
+// https://github.com/gleam-lang/gleam/issues/3379
+#[test]
+fn single_clause_variables() {
+    assert_js!(
+        r#"
+pub fn main() {
+  let text = "first defined"
+  case "defined again" {
+    text -> Nil
+  }
+  let text = "a third time"
+}
+"#
+    )
+}
+
+// https://github.com/gleam-lang/gleam/issues/3379
+#[test]
+fn single_clause_variables_assigned() {
+    assert_js!(
+        r#"
+pub fn main() {
+  let text = "first defined"
+  let other = case "defined again" {
+    text -> Nil
+  }
+  let text = "a third time"
+}
+"#
+    )
+}
