@@ -7,7 +7,7 @@ mod hover;
 mod signature_help;
 
 use std::{
-    collections::HashMap,
+    collections::{HashMap, HashSet},
     sync::{Arc, Mutex},
     time::SystemTime,
 };
@@ -22,13 +22,14 @@ use lsp_types::{Position, TextDocumentIdentifier, TextDocumentPositionParams, Ur
 use crate::{
     config::PackageConfig,
     io::{
-        memory::InMemoryFileSystem, CommandExecutor, FileSystemReader, FileSystemWriter, ReadDir,
-        WrappedReader,
+        memory::InMemoryFileSystem, BeamCompiler, CommandExecutor, FileSystemReader,
+        FileSystemWriter, ReadDir, WrappedReader,
     },
     language_server::{
         engine::LanguageServerEngine, files::FileSystemProxy, progress::ProgressReporter,
         DownloadDependencies, LockGuard, Locker, MakeLocker,
     },
+    line_numbers::LineNumbers,
     manifest::{Base16Checksum, Manifest, ManifestPackage, ManifestPackageSource},
     paths::ProjectPaths,
     requirement::Requirement,
@@ -223,9 +224,21 @@ impl CommandExecutor for LanguageServerTestIO {
         cwd: Option<&Utf8Path>,
         stdio: crate::io::Stdio,
     ) -> Result<i32> {
+        panic!("exec({program:?}, {args:?}, {env:?}, {cwd:?}, {stdio:?}) is not implemented")
+    }
+}
+
+impl BeamCompiler for LanguageServerTestIO {
+    fn compile_beam(
+        &self,
+        out: &Utf8Path,
+        lib: &Utf8Path,
+        modules: &HashSet<Utf8PathBuf>,
+        stdio: crate::io::Stdio,
+    ) -> Result<()> {
         panic!(
-            "exec({:?}, {:?}, {:?}, {:?}, {:?}) is not implemented",
-            program, args, env, cwd, stdio
+            "compile_beam({:?}, {:?}, {:?}, {:?}) is not implemented",
+            out, lib, modules, stdio
         )
     }
 }
@@ -541,9 +554,9 @@ impl<'a> TestProject<'a> {
         test_name: &str,
     ) -> TextDocumentPositionParams {
         let path = Utf8PathBuf::from(if cfg!(target_family = "windows") {
-            format!(r"\\?\C:\test\{}.gleam", test_name)
+            format!(r"\\?\C:\test\{test_name}.gleam")
         } else {
-            format!("/test/{}.gleam", test_name)
+            format!("/test/{test_name}.gleam")
         });
 
         let url = Url::from_file_path(path).unwrap();
@@ -721,3 +734,26 @@ fn byte_index_to_position(src: &str, byte_index: usize) -> Position {
 
     Position::new(line, col)
 }
+
+/// This function replicates how the text editor applies TextEdit.
+///
+pub fn apply_code_edit(src: &str, mut change: Vec<lsp_types::TextEdit>) -> String {
+    let mut result = src.to_string();
+    let line_numbers = LineNumbers::new(src);
+    let mut offset = 0;
+
+    change.sort_by_key(|edit| (edit.range.start.line, edit.range.start.character));
+    for edit in change {
+        let start = line_numbers.byte_index(edit.range.start.line, edit.range.start.character)
+            as i32
+            - offset;
+        let end =
+            line_numbers.byte_index(edit.range.end.line, edit.range.end.character) as i32 - offset;
+        let range = (start as usize)..(end as usize);
+        offset += end - start;
+        offset -= edit.new_text.len() as i32;
+        result.replace_range(range, &edit.new_text);
+    }
+
+    result
+}
