@@ -9,7 +9,6 @@
     clippy::needless_continue,
     clippy::needless_borrow,
     clippy::match_wildcard_for_single_variants,
-    clippy::mismatched_target_os,
     clippy::match_on_vec_items,
     clippy::imprecise_flops,
     clippy::suboptimal_flops,
@@ -28,6 +27,7 @@
     trivial_casts,
     trivial_numeric_casts,
     nonstandard_style,
+    unexpected_cfgs,
     unused_import_braces,
     unused_qualifications,
 )]
