@@ -1,3 +1,5 @@
+use ecow::EcoString;
+
 use crate::ast::SrcSpan;
 use crate::parse::error::{LexicalError, LexicalErrorType};
 use crate::parse::token::Token;
@@ -563,11 +565,13 @@ where
             })
         } else {
             let value = format!("{prefix}{num}");
+            let int_value = super::parse_int_value(&value).expect("int value to parse as bigint");
             let end_pos = self.get_pos();
             Ok((
                 start_pos,
                 Token::Int {
                     value: value.into(),
+                    int_value,
                 },
                 end_pos,
             ))
@@ -619,11 +623,13 @@ where
                 end_pos,
             )
         } else {
+            let int_value = super::parse_int_value(&value).expect("int value to parse as bigint");
             let end_pos = self.get_pos();
             (
                 start_pos,
                 Token::Int {
                     value: value.into(),
+                    int_value,
                 },
                 end_pos,
             )
@@ -705,7 +711,7 @@ where
             }
             _ => Kind::Comment,
         };
-        let mut content = String::new();
+        let mut content = EcoString::new();
         let start_pos = self.get_pos();
         while Some('\n') != self.chr0 {
             match self.chr0 {
