@@ -19,6 +19,7 @@ pub(crate) use expression::ExprTyper;
 pub use fields::FieldMap;
 use hexpm::version::Version;
 pub use prelude::*;
+use printer::Names;
 use serde::Serialize;
 
 use crate::{
@@ -62,6 +63,34 @@ pub enum Type {
         module: EcoString,
         name: EcoString,
         args: Vec<Arc<Type>>,
+
+        /// Which variant of the types this value is, if it is known from variant inference.
+        /// This allows us to permit certain operations when we know this,
+        /// such as record updates for multi-constructor types, or field access
+        /// for fields not shared between type variants. For example:
+        ///
+        /// ```gleam
+        /// type Wibble {
+        ///   Wibble(wibble: Int, other, Int)
+        ///   Wobble(wobble: Int, something: Int)
+        /// }
+        ///
+        /// fn add_one(some_wibble: Wibble) -> Wibble {
+        ///   case some_wibble {
+        ///     Wibble(..) as wibble -> Wibble(..wibble, other: wibble.other + 1)
+        ///     Wobble(..) as wobble -> Wobble(..wobble, something: wobble.something + 1)
+        ///   }
+        /// }
+        /// ```
+        ///
+        /// Here, the `wibble` variable has an inferred variant of `0`, since we know it's
+        /// of the `Wibble` variant. This means we can safely update it using the `Wibble`
+        /// constructor, and access the `other` field, which is only present in that variant.
+        ///
+        /// However, the parameter `some_wibble` has no known variant; it could be either of the variants,
+        /// so we can't allow any of that until we pattern match on it.
+        ///
+        inferred_variant: Option<u16>,
     },
 
     /// The type of a function. It takes arguments and returns a value.
@@ -99,6 +128,16 @@ impl Type {
         }
     }
 
+    pub fn result_ok_type(&self) -> Option<Arc<Type>> {
+        match self {
+            Self::Named {
+                module, name, args, ..
+            } if "Result" == name && is_prelude_module(module) => args.first().cloned(),
+            Self::Var { type_ } => type_.borrow().result_ok_type(),
+            Self::Named { .. } | Self::Tuple { .. } | Type::Fn { .. } => None,
+        }
+    }
+
     pub fn is_unbound(&self) -> bool {
         match self {
             Self::Var { type_ } => type_.borrow().is_unbound(),
@@ -225,6 +264,46 @@ impl Type {
         }
     }
 
+    pub fn set_custom_type_variant(&mut self, index: u16) {
+        match self {
+            Type::Named {
+                inferred_variant, ..
+            } => *inferred_variant = Some(index),
+            Type::Var { type_ } => type_.borrow_mut().set_custom_type_variant(index),
+            Type::Fn { .. } | Type::Tuple { .. } => {}
+        }
+    }
+
+    pub fn generalise_custom_type_variant(&mut self) {
+        match self {
+            Type::Named {
+                inferred_variant, ..
+            } => *inferred_variant = None,
+            Type::Var { type_ } => type_.borrow_mut().generalise_custom_type_variant(),
+            Type::Tuple { elems } => {
+                for element in elems {
+                    Arc::make_mut(element).generalise_custom_type_variant();
+                }
+            }
+            Type::Fn { args, retrn } => {
+                for argument in args {
+                    Arc::make_mut(argument).generalise_custom_type_variant();
+                }
+                Arc::make_mut(retrn).generalise_custom_type_variant();
+            }
+        }
+    }
+
+    pub fn custom_type_inferred_variant(&self) -> Option<u16> {
+        match self {
+            Type::Named {
+                inferred_variant, ..
+            } => *inferred_variant,
+            Type::Var { type_ } => type_.borrow().custom_type_inferred_variant(),
+            Type::Fn { .. } | Type::Tuple { .. } => None,
+        }
+    }
+
     /// Get the args for the type if the type is a specific `Type::App`.
     /// Returns None if the type is not a `Type::App` or is an incorrect `Type:App`
     ///
@@ -283,6 +362,7 @@ impl Type {
                         module: module.into(),
                         args: args.clone(),
                         publicity,
+                        inferred_variant: None,
                     }),
                 };
                 Some(args)
@@ -357,7 +437,19 @@ pub fn collapse_links(t: Arc<Type>) -> Arc<Type> {
 pub struct AccessorsMap {
     pub publicity: Publicity,
     pub type_: Arc<Type>,
-    pub accessors: HashMap<EcoString, RecordAccessor>,
+    pub shared_accessors: HashMap<EcoString, RecordAccessor>,
+    pub variant_specific_accessors: Vec<HashMap<EcoString, RecordAccessor>>,
+}
+
+impl AccessorsMap {
+    pub fn accessors_for_variant(
+        &self,
+        inferred_variant: Option<u16>,
+    ) -> &HashMap<EcoString, RecordAccessor> {
+        inferred_variant
+            .and_then(|index| self.variant_specific_accessors.get(index as usize))
+            .unwrap_or(&self.shared_accessors)
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -407,8 +499,8 @@ pub enum ValueConstructorVariant {
         field_map: Option<FieldMap>,
         location: SrcSpan,
         module: EcoString,
-        constructors_count: u16,
-        constructor_index: u16,
+        variants_count: u16,
+        variant_index: u16,
         documentation: Option<EcoString>,
     },
 }
@@ -536,6 +628,16 @@ impl ValueConstructorVariant {
             } => *implementations,
         }
     }
+
+    fn record_field_map(&self) -> Option<&FieldMap> {
+        match self {
+            ValueConstructorVariant::LocalVariable { .. }
+            | ValueConstructorVariant::ModuleConstant { .. }
+            | ValueConstructorVariant::LocalConstant { .. }
+            | ValueConstructorVariant::ModuleFn { .. } => None,
+            ValueConstructorVariant::Record { field_map, .. } => field_map.as_ref(),
+        }
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -829,6 +931,13 @@ impl TypeVar {
         }
     }
 
+    pub fn custom_type_inferred_variant(&self) -> Option<u16> {
+        match self {
+            Self::Link { type_ } => type_.custom_type_inferred_variant(),
+            Self::Unbound { .. } | Self::Generic { .. } => None,
+        }
+    }
+
     pub fn is_result(&self) -> bool {
         match self {
             Self::Link { type_ } => type_.is_result(),
@@ -836,6 +945,13 @@ impl TypeVar {
         }
     }
 
+    pub fn result_ok_type(&self) -> Option<Arc<Type>> {
+        match self {
+            TypeVar::Link { type_ } => type_.result_ok_type(),
+            TypeVar::Unbound { .. } | TypeVar::Generic { .. } => None,
+        }
+    }
+
     pub fn fn_types(&self) -> Option<(Vec<Arc<Type>>, Arc<Type>)> {
         match self {
             Self::Link { type_ } => type_.fn_types(),
@@ -892,6 +1008,20 @@ impl TypeVar {
             Self::Unbound { .. } | Self::Generic { .. } => None,
         }
     }
+
+    pub fn set_custom_type_variant(&mut self, index: u16) {
+        match self {
+            Self::Link { type_ } => Arc::make_mut(type_).set_custom_type_variant(index),
+            Self::Unbound { .. } | Self::Generic { .. } => {}
+        }
+    }
+
+    pub fn generalise_custom_type_variant(&mut self) {
+        match self {
+            Self::Link { type_ } => Arc::make_mut(type_).generalise_custom_type_variant(),
+            Self::Unbound { .. } | Self::Generic { .. } => {}
+        }
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -1133,6 +1263,7 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
             package,
             name,
             args,
+            inferred_variant: _,
         } => {
             let args = args.iter().map(|t| generalise(t.clone())).collect();
             Arc::new(Type::Named {
@@ -1141,6 +1272,7 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
                 package: package.clone(),
                 name: name.clone(),
                 args,
+                inferred_variant: None,
             })
         }
 
@@ -1157,6 +1289,12 @@ pub fn generalise(t: Arc<Type>) -> Arc<Type> {
 pub enum FieldAccessUsage {
     /// Used as `thing.field()`
     MethodCall,
+    /// Used internally when trying to access a field when performing record updates.
+    /// The `infer_record_update` function uses this to determine which fields exist
+    /// on a certain record to know whether a certain record update updates correct fields.
+    /// Without this distinction, we get confusing error messages in certain cases.
+    ///
+    RecordUpdate,
     /// Used as `thing.field`
     Other,
 }
