@@ -1,11 +1,12 @@
 use camino::{Utf8Path, Utf8PathBuf};
 use gleam_core::{
     io::{
-        memory::InMemoryFileSystem, CommandExecutor, FileSystemReader, FileSystemWriter, ReadDir,
-        Stdio, WrappedReader,
+        memory::InMemoryFileSystem, BeamCompiler, CommandExecutor, FileSystemReader,
+        FileSystemWriter, ReadDir, Stdio, WrappedReader,
     },
     Error, Result,
 };
+use std::collections::HashSet;
 
 #[derive(Clone, Debug, Default)]
 pub struct WasmFileSystem {
@@ -31,6 +32,18 @@ impl CommandExecutor for WasmFileSystem {
     }
 }
 
+impl BeamCompiler for WasmFileSystem {
+    fn compile_beam(
+        &self,
+        _out: &Utf8Path,
+        _lib: &Utf8Path,
+        _modules: &HashSet<Utf8PathBuf>,
+        _stdio: Stdio,
+    ) -> Result<(), Error> {
+        Ok(()) // Always succeed.
+    }
+}
+
 impl FileSystemWriter for WasmFileSystem {
     fn delete_directory(&self, path: &Utf8Path) -> Result<(), Error> {
         tracing::trace!("delete {:?}", path);
